@@ -219,8 +219,8 @@
 //! general-purpose parsers, like the `Zip`
 //! decompressor.
 //!
-//! [`BufRead`]: https://doc.rust-lang.org/stable/std/io/trait.BufRead.html
-//! [`BufReader`]: https://doc.rust-lang.org/stable/std/io/struct.BufReader.html
+//! [`BufRead`]: std::io::BufRead
+//! [`BufReader`]: std::io::BufReader
 //! [HTTP's chunk transfer encoding]: https://en.wikipedia.org/wiki/Chunked_transfer_encoding
 
 #![doc(html_favicon_url = "https://docs.sequoia-pgp.org/favicon.png")]
@@ -301,6 +301,15 @@ fn vec_truncate(v: &mut Vec<u8>, len: usize) {
     }
 }
 
+/// Like `Vec<u8>::resize`, but fast in debug builds.
+fn vec_resize(v: &mut Vec<u8>, new_size: usize) {
+    if v.len() < new_size {
+        v.resize(new_size, 0);
+    } else {
+        vec_truncate(v, new_size);
+    }
+}
+
 /// The generic `BufferReader` interface.
 pub trait BufferedReader<C> : io::Read + fmt::Debug + fmt::Display + Send + Sync
   where C: fmt::Debug + Send + Sync
@@ -402,7 +411,7 @@ pub trait BufferedReader<C> : io::Read + fmt::Debug + fmt::Display + Send + Sync
                                       "unexpected EOF"));
             }
         }
-        return result;
+        result
     }
 
     /// Returns all of the data until EOF.  Like `data()`, this does not
@@ -446,7 +455,7 @@ pub trait BufferedReader<C> : io::Read + fmt::Debug + fmt::Display + Send + Sync
         // must return a slice shorter than std::usize::MAX.
         loop {
             match self.data(s) {
-                Ok(ref buffer) => {
+                Ok(buffer) => {
                     if buffer.len() < s {
                         // We really want to do
                         //
@@ -472,7 +481,7 @@ pub trait BufferedReader<C> : io::Read + fmt::Debug + fmt::Display + Send + Sync
 
         let buffer = self.buffer();
         assert_eq!(buffer.len(), s);
-        return Ok(buffer);
+        Ok(buffer)
     }
 
     /// Consumes some of the data.
@@ -549,7 +558,7 @@ pub trait BufferedReader<C> : io::Read + fmt::Debug + fmt::Display + Send + Sync
     /// }
     ///
     /// // Like `data()`, `data_consume()` may return and consume less
-    /// // than request if there is no more data available.
+    /// // than requested if there is no more data available.
     /// {
     ///     let buffer = br.data_consume(10)?;
     ///     assert_eq!(buffer, &orig[6..6 + buffer.len()]);
@@ -603,7 +612,6 @@ pub trait BufferedReader<C> : io::Read + fmt::Debug + fmt::Display + Send + Sync
     /// iff the end of the stream is reached, but the reader is not
     /// consummated.
     ///
-    /// [`Limitor`]: struct.Limitor.html
     fn consummated(&mut self) -> bool {
         self.eof()
     }
@@ -723,13 +731,13 @@ pub trait BufferedReader<C> : io::Read + fmt::Debug + fmt::Display + Send + Sync
             let len = {
                 // Try self.buffer.  Only if it is empty, use
                 // self.data.
-                let buffer = if self.buffer().len() == 0 {
+                let buffer = if self.buffer().is_empty() {
                     self.data(DEFAULT_BUF_SIZE)?
                 } else {
                     self.buffer()
                 };
 
-                if buffer.len() == 0 {
+                if buffer.is_empty() {
                     break 'outer 0;
                 }
 
@@ -747,7 +755,7 @@ pub trait BufferedReader<C> : io::Read + fmt::Debug + fmt::Display + Send + Sync
         };
 
         self.consume(position);
-        return Ok(total + position);
+        Ok(total + position)
     }
 
     /// Discards the input until one of the bytes in `terminals` is
@@ -785,7 +793,7 @@ pub trait BufferedReader<C> : io::Read + fmt::Debug + fmt::Display + Send + Sync
         if data.len() > amount {
             data = &data[..amount];
         }
-        return Ok(data.to_vec());
+        Ok(data.to_vec())
     }
 
     /// Like `steal()`, but instead of stealing a fixed number of
@@ -793,7 +801,7 @@ pub trait BufferedReader<C> : io::Read + fmt::Debug + fmt::Display + Send + Sync
     fn steal_eof(&mut self) -> Result<Vec<u8>, std::io::Error> {
         let len = self.data_eof()?.len();
         let data = self.steal(len)?;
-        return Ok(data);
+        Ok(data)
     }
 
     /// Like `steal_eof()`, but instead of returning the data, the
@@ -835,12 +843,13 @@ pub trait BufferedReader<C> : io::Read + fmt::Debug + fmt::Display + Send + Sync
                 writeln!(sink, "  {}. {}, {:?}", i, r, cookie)?;
             }
             reader = r.get_ref();
-            i = i + 1;
+            i += 1;
         }
         Ok(())
     }
 
     /// Boxes the reader.
+    #[allow(clippy::wrong_self_convention)]
     fn as_boxed<'a>(self) -> Box<dyn BufferedReader<C> + 'a>
         where Self: 'a + Sized
     {
@@ -912,14 +921,13 @@ pub trait BufferedReader<C> : io::Read + fmt::Debug + fmt::Display + Send + Sync
 /// implementations of trait `std::io::Read` for type `&mut _`").
 pub fn buffered_reader_generic_read_impl<T: BufferedReader<C>, C: fmt::Debug + Sync + Send>
         (bio: &mut T, buf: &mut [u8]) -> Result<usize, io::Error> {
-    match bio.data_consume(buf.len()) {
-        Ok(inner) => {
+    bio
+        .data_consume(buf.len())
+        .map(|inner| {
             let amount = cmp::min(buf.len(), inner.len());
             buf[0..amount].copy_from_slice(&inner[0..amount]);
-            return Ok(amount);
-        },
-        Err(err) => return Err(err),
-    }
+            amount
+        })
 }
 
 /// Make a `Box<BufferedReader>` look like a BufferedReader.
@@ -1097,7 +1105,7 @@ mod test {
             {
                 let result = bio.data(buffer.len());
                 let buffer = result.unwrap();
-                if buffer.len() > 0 {
+                if !buffer.is_empty() {
                     assert_eq!(buffer,
                                &data[data_start..data_start + buffer.len()]);
                 }
