@@ -66,7 +66,7 @@ impl<T: BufferedReader<C>, C: fmt::Debug + Sync + Send> io::Read for Reserve<T,
 
         let to_read = cmp::min(buf.len(), to_read);
 
-        return self.reader.read(&mut buf[..to_read]);
+        self.reader.read(&mut buf[..to_read])
     }
 }
 
